<?php
namespace Teknisa\PLA\Modules\Mymeals\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Planejamento\Entities\Confirmarefe;
use Teknisa\Model\Planejamento\Entities\Compcard;
use Teknisa\Model\Planejamento\Entities\Cardacli;
use Teknisa\Model\Planejamento\Entities\Cardadia;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Services\SMTPService;
use Teknisa\Libs\Util\SMTP;
use Teknisa\PLA\Util\UtilSMTP;

class Pla02030AprovacaoComensaisService {

    protected $entityManager;
    protected $environment;
    protected $dataSourceManager;
    protected $smtpService;

    public function __construct(EntityManager $entityManager, $environment, DataSource\Manager $dataSourceManager)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->templatesDir = __DIR__.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'Templates'.DIRECTORY_SEPARATOR;
        $this->dataSourceManager = $dataSourceManager;
        $this->smtpService = new SMTPService(array());
    }

    public function iniciaTransacao(){
        $this->entityManager->beginTransaction();
    }

    public function commit(){
        $this->entityManager->commit();
    }

    public function flush(){
        $this->entityManager->flush();
    }

    public function rollback(){
        $this->entityManager->rollback();
    }
    
    public function atualizaCardadia($row){
        $cardadia = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cardadia')
            ->findOneBy(
                array(
                    "nrorg" => $row['NRORG'],
                    "cdfilial" => $row['CDFILIAL'],
                    "nrcardapio" => $row['NRCARDAPIO']
                )
            );
            
        if($cardadia instanceof Cardadia){
            $cardadia->setNrorg($row['NRORG']);
            $cardadia->setCdfilial($row['CDFILIAL']);
            $cardadia->setNrcardapio($row['NRCARDAPIO']);
            $cardadia->setQtcomesticar($row['QTCOMESTICAR']);
            $cardadia->setQtcliesticar($row['QTCLIESTICAR']);
            $cardadia->setQtfunesticar($row['QTFUNESTICAR']);
            $cardadia->setQtdoaesticar($row['QTDOAESTICAR']);
            $cardadia->setQtestiturno1($row['QTESTITURNO1']);
            $cardadia->setQtestiturno2($row['QTESTITURNO2']);
            $cardadia->setQtestiturno3($row['QTESTITURNO3']);
            $cardadia->setQtestiturno4($row['QTESTITURNO4']);
                
            $this->entityManager->persist($cardadia);
        }
    }
    
    public function atualizaComensaisCardacli($row){
        $cardacli = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cardacli')
            ->findOneBy(
                array(
                    "nrorg" => $row['NRORG'],
                    "cdcliente" => $row['CDCLIENTE'],
                    "nrcardapio" => $row['NRCARDAPIO']
                )
            );
                
        if($cardacli instanceof Cardadia){
            $cardacli->setNrorg($row['NRORG']);
            $cardacli->setNrcardapio($row['NRCARDAPIO']);
            $cardacli->setCdcliente($row['CDCLIENTE']);
            $cardacli->setCddestfili($row['CDDESTFILI']);
            $cardacli->setQtcomealterada($row['QTCOMEALTERADA']);
                
            $this->entityManager->persist($cardacli);
        }
    }
    
    public function getCardadiaAprov($params){
        try{
            return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cardadia')
                ->findOneBy(
                    array(
                        "cdfilial" => $params['CDFILIAL'],
                        "nrcardapio" => $params['NRCARDAPIO']
                    )
                );
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getAlteracoesCardapio($params){
        $sql = "SELECT CO.* FROM CONFIRMAREFE CO INNER JOIN(SELECT NRORG, CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO, MAX(NRCONFIRMAREFE) AS NRCONFIRMAREFE FROM CONFIRMAREFE WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND TO_CHAR(DTCARDAPIO,'DD/MM/YYYY') = :DTCARDAPIO AND NRCARDAPIO = :NRCARDAPIO AND CDCLIENTE IS NULL AND CDDESTFILI IS NULL AND CDTURNO IS NULL AND IDQTDEAPROVADA = :IDQTDEAPROVADA GROUP BY NRORG, CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO) C ON C.NRCONFIRMAREFE = CO.NRCONFIRMAREFE ORDER BY CO.NRCONFIRMAREFE, CO.CDFILIAL, CO.CDSERVFILI, CO.NRCARDAPIO, CO.DTCARDAPIO";
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getAlteracoesTurno($params){
        $sql = "SELECT CO.* FROM CONFIRMAREFE CO INNER JOIN(SELECT NRORG, CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO, CDTURNO, MAX(NRCONFIRMAREFE) AS NRCONFIRMAREFE FROM CONFIRMAREFE WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND NRCARDAPIO = :NRCARDAPIO AND TO_CHAR(DTCARDAPIO,'DD/MM/YYYY') = :DTCARDAPIO AND CDCLIENTE IS NULL AND CDDESTFILI IS NULL AND CDTURNO IS NOT NULL AND IDQTDEAPROVADA = :IDQTDEAPROVADA GROUP BY NRORG, CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO, CDTURNO) C ON C.NRCONFIRMAREFE = CO.NRCONFIRMAREFE ORDER BY CO.NRCONFIRMAREFE, CO.CDFILIAL, CO.CDSERVFILI, CO.NRCARDAPIO, CO.DTCARDAPIO";
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getAlteracoesPontoEntrega($params){
        $sql = "SELECT CO.* FROM CONFIRMAREFE CO INNER JOIN(SELECT NRORG, CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO, CDCLIENTE, CDDESTFILI, MAX(NRCONFIRMAREFE) AS NRCONFIRMAREFE FROM CONFIRMAREFE WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND NRCARDAPIO = :NRCARDAPIO AND TO_CHAR(DTCARDAPIO,'DD/MM/YYYY') = :DTCARDAPIO AND CDCLIENTE IS NOT NULL AND CDDESTFILI IS NOT NULL AND CDTURNO IS NULL AND IDQTDEAPROVADA = :IDQTDEAPROVADA GROUP BY NRORG, CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO, CDCLIENTE, CDDESTFILI) C ON C.NRCONFIRMAREFE = CO.NRCONFIRMAREFE ORDER BY CO.NRCONFIRMAREFE, CO.CDFILIAL, CO.CDSERVFILI, CO.NRCARDAPIO, CO.DTCARDAPIO";
        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insereConfirmarefe($row){
        $datetime = new \DateTime();
        $conditions = array(
                array('nrorg' => $row['NRORG']),
                array('nrconfirmarefe' => $row['NRCONFIRMAREFE'])
            );
        
        $confirmarefe = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Confirmarefe')
        ->findOneBy(array('nrconfirmarefe' => $row['NRCONFIRMAREFE']), array('nrconfirmarefe' => "DESC"));
        
        
        if ($confirmarefe instanceof Confirmarefe) {

            $confirmarefe->setQtcomeoriginal($row['QTCOMEORIGINAL']);
            $confirmarefe->setQtcomealterada($row['QTCOMEALTERADA']);
            $confirmarefe->setQttotalalterada($row['QTTOTALALTERADA']);
            $confirmarefe->setIdqtdeaprovada($row['IDQTDEAPROVADA']);
            $confirmarefe->setDsparametros($row['DSPARAMETROS']);
            $confirmarefe->setNrorgultatu($row['NRORG']);
            $confirmarefe->setDtultatu(new \DateTime());
            $confirmarefe->setCdoperultatu($row['CDOPERADOR']);
            $this->entityManager->persist($confirmarefe);
        }
    }
    
    public function aprovaCardapio($row){
        $confirmarefe = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Confirmarefe')
        ->findBy(array('nrorg' => $row['NRORG'],'cdfilial' => $row['CDFILIAL'],
        'cdservfili' => $row['CDSERVFILI'],'nrcardapio' => $row['NRCARDAPIO'], 'idqtdeaprovada' => $row['IDQTDEAPROVADA']));
       
        foreach ($confirmarefe as $confirmarefeItem) {
            if($confirmarefeItem instanceof Confirmarefe) {
                $confirmarefeItem->setIdqtdeaprovada('APROVADA');
                $confirmarefeItem->setNrorgultatu($row['NRORG']);
                $confirmarefeItem->setDtultatu(new \DateTime());
                $confirmarefeItem->setCdoperultatu($row['CDOPERADOR']);
                $this->entityManager->persist($confirmarefeItem);
            }
        }
    }
    
    public function reprovaCardapio($row){
        try{
            $this->iniciaTransacao();
            $confirmarefe = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Confirmarefe')
            ->findBy(array('nrorg' => $row['NRORG'], 'cdfilial' => $row['CDFILIAL'],
            'cdservfili' => $row['CDSERVFILI'],'nrcardapio' => $row['NRCARDAPIO'], 'idqtdeaprovada' => $row['IDQTDEAPROVADA']));
            foreach ($confirmarefe as $confirmarefeItem) {
                if($confirmarefeItem instanceof Confirmarefe) {
                    $confirmarefeItem->setIdqtdeaprovada('REPROVADA');
                    $confirmarefeItem->setNrorgultatu($row['NRORG']);
                    $confirmarefeItem->setDtultatu(new \DateTime());
                    $confirmarefeItem->setCdoperultatu($row['CDOPERADOR']);
                
                    $this->entityManager->persist($confirmarefeItem);
                } 
            }
            $this->commit();
            
        }catch(\Exception $e) {
            $this->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function salvaReprovacaoHistoricoMymeals($dados) {
        $query = "UPDATE HISTORICOMYMEALS SET IDSITUACAO = 'REPROVADO' , DSJUSTIFICATIVA = :JUSTIFICATIVA WHERE NRHISTORICOMYMEALS = :NRHISTORICOMYMEALS";
        $params = array(
            "NRHISTORICOMYMEALS" => $dados['NRHISTORICOMYMEALS'],
            "JUSTIFICATIVA" => $dados['JUSTIFICATIVA']
        );
        $this->entityManager->getConnection()->prepare($query)->execute($params);       
    }
    
    public function salvaAprovacaoHistoricoMymeals($dados) {
        $query = "UPDATE HISTORICOMYMEALS SET IDSITUACAO = 'APROVADO' WHERE NRHISTORICOMYMEALS = :NRHISTORICOMYMEALS";
        $params = array(
            "NRHISTORICOMYMEALS" => $dados['NRHISTORICOMYMEALS']
        );
        $this->entityManager->getConnection()->prepare($query)->execute($params);       
    }
    
    public function verificaEfetivos($row) {
        $sql = "SELECT DISTINCT S.CDFILIAL, F.NMFILIAL, S.CDSERVFILI, S.NMSERVICO, E.DTEFETCLIE, E.NRAPURACFAT
                  FROM EFETCLIE E
                       LEFT JOIN FILIAL F ON F.CDFILIAL = E.CDFILIAL
                       LEFT JOIN SERVICO S ON S.CDFILIAL = E.CDFILIAL AND
                                              S.CDSERVFILI = E.CDSERVPLAN
                 WHERE E.DTEFETCLIE = :DTEFETCLIE
                    AND E.CDFILIAL = :CDFILIAL
                    AND E.CDSERVFILI = :CDSERVFILI";
                   
        $params = array(
            "DTEFETCLIE" => $row['DTCARDAPIO'],
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI']
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
    }
    
    public function excluirCardapio($cardapio, $emailMessages) {
        try {
            $tipoCardServ = $this->verificaTipoCardapioPorServico($cardapio['CDFILIAL'], $cardapio['CDSERVFILI']);
        
            $cardadiaComDitribuicao = $this->validaDistriuicaoCardapio($cardapio);

            if(!$cardadiaComDitribuicao){
        
                $notification = "";
                $houveExclusao = [];
   
                $possuiRetirada = $this->verificaRetirada($cardapio['CDFILIAL'], $cardapio['NRCARDAPIO']);

                if(empty($possuiRetirada)){
                    
                    $modoProc = $this->getModoProc();
                    if($modoProc === 'NOT_FOUNDED_GROUP_OPER'){ 
                        /* Foi definido que quando não há grupo cadastrado
                            para alterar o cardápio, o mesmo deve executar os SELECT's / 
                            operações do Cardápio tipo Livre
                        */
                        $tipoCardServ = "N";
                    }
                    
                    $this->checkCardToDelete($modoProc, $cardapio, $tipoCardServ);
                    $this->deleteCompCard($cardapio);
                    $this->excluirCardadia($cardapio);
                    $this->excluirCardacli($cardapio);
                   
                    $dadosAprovaCardapio = array(
                        "NRORG" => $this->environment->getNrOrg(),
                        "NRCONFIRMAREFE" => $cardapio['NRCONFIRMAREFE'],
                        "CDFILIAL" => $cardapio['CDFILIAL'],
                        "CDSERVFILI" => $cardapio['CDSERVFILI'],
                        "NRCARDAPIO" => $cardapio['NRCARDAPIO'],
                        "DTCARDAPIO" => $cardapio['DTCARDAPIO'],
                        "IDQTDEAPROVADA" => 'PENDENTE',
                        "NRHISTORICOMYMEALS" => $cardapio['NRHISTORICOMYMEALS'],
                        "CDOPERADOR" => $this->environment->getCdOperador(),
                        "NMFILIAL" => $cardapio['NMFILIAL'],
                        "NMOPERADOR" => $cardapio['NMOPERADOR'],
                        "CDOPERULTATU" => $cardapio['CDOPERULTATU'],
                        "NMSERVICO" => $cardapio['NMSERVICO'],
                        "QTCOMEALTERADA" => $cardapio['QTCOMEALTERADA'],
                        "QTCOMEORIGINAL" => $cardapio['QTCOMEORIGINAL']
                    );
                    
                    try {
                        $this->iniciaTransacao();
                        $this->aprovaCardapio($dadosAprovaCardapio);
                        $this->commit();
                    }catch(\Exception $e) {
                        $this->rollback();
                        $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
                    }
                    $this->flush();
                    $this->salvaAprovacaoHistoricoMymeals($dadosAprovaCardapio);
                    $emailMessages["emailDescription"] = str_replace("__TYPE__", "", $emailMessages["emailDescription"]);

                    $this->sendMail($dadosAprovaCardapio, $emailMessages, 'APROVADO');
                    
                  
                    return [
                        "SITUACAO" => 'EXCLUIDO_COM_SUCESSO',
                        "CDFILIAL" => $cardapio['CDFILIAL'],
                        "CDSERVFILI" => $cardapio['CDSERVFILI'],
                        "NMSERVICO" => $cardapio['NMSERVICO'],
                        "DTCARDAPIO" => $cardapio['DTCARDAPIO']
                    ];
                } else {
                    return [
                        "SITUACAO" => 'POSSUI_RETIRADA',
                        "CDFILIAL" => $cardapio['CDFILIAL'],
                        "CDSERVFILI" => $cardapio['CDSERVFILI'],
                        "NMSERVICO" => $cardapio['NMSERVICO'],
                        "DTCARDAPIO" => $cardapio['DTCARDAPIO']
                    ];
                }
            
            }else{
                return [
                    "SITUACAO" => 'POSSUI_DISTRIBUICAO',
                    "CDFILIAL" => $cardapio['CDFILIAL'],
                    "CDSERVFILI" => $cardapio['CDSERVFILI'],
                    "NMSERVICO" => $cardapio['NMSERVICO'],
                    "DTCARDAPIO" => $cardapio['DTCARDAPIO']
                ];
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function dependeLiberacao($tipoCard, $cdfilial, $cdservfili) {
        if($tipoCard === 'D') {
            $query = "SELECT
                        NVL(IDCARDDESCENTR, 'N') AS IDCARDDESCENTR
                    FROM 
                        SERVICO
                    WHERE
                        CDFILIAL = :CDFILIAL AND
                        CDSERVFILI = :CDSERVFILI";
            $params = array(
                "CDFILIAL" => $cdfilial,
                "CDSERVFILI" => $cdservfili
            );
            $result = $this->entityManager->getConnection()->fetchAll($query, $params);
            return $result[0]['IDCARDDESCENTR'] === "S";
        } else {
            return false;
        }
    }
    
    private function getModoProc() {
        $filterCriteria = new FilterCriteria('opergrupop');
        $filterCriteria->addCondition('CDOPERADOR', $this->environment->getCdOperador());
        $filterCriteria->addCondition('CDGRUPOPER', $this->getGrupOper());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        return $this->getGrupOper() ? (isset($rows[0]) ? 'M' : 'F') : 'NOT_FOUNDED_GROUP_OPER';
    }
    
    private function getGrupOper() {
        $filterCriteria = new FilterCriteria('autgrupop');
        $filterCriteria->addCondition('IDAUTGRUPOP', 'LRCD');
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        return isset($rows[0]) ? $rows[0]['CDGRUPOPER'] : null;
    }
    
    private function verificaCardapioLiberado($row) {
        $query = "SELECT
                    CDFILIAL,
                    DTLIBECARDMATR
                FROM
                    CTRLAPROVCARD
                WHERE
                    CDFILIAL =:CDFILIAL AND
                    CDSERVFILI =:CDSERVFILI AND
                    DTMESCARDAPROV =:DTMESCARDAPROV AND
                    DTLIBECARDMATR IS NOT NULL";
        $dtmescardaprov = date('01/m/Y', strtotime(str_replace('/', '-', $row['DTCARDAPIO'])));
        $params = array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTMESCARDAPROV" => $row['DTCARDAPIO']
        );
        $result = $this->entityManager->getConnection()->fetchAll($query, $params);
        if(sizeof($result)) {
            return 'TODOS';
        } else {
            return 'N';
        }
    }
    
    private function getPratosEspeCard($cardLib, $cdfilial, $nrcardapio) {
        $query = "SELECT
                    NRSEQPRATCAR,
                    IDPRATOESPE,
                    CDFILAUXPRAT,
                    CDPRATO,
                    CDFILAUXPRATMAT,
                    CDPRATOMAT
                FROM
                    COMPCARD
                WHERE
                    IDPRATOESPE <>:IDPRATOESPE AND
                    CDFILIAL =:CDFILIAL AND
                    NRCARDAPIO =:NRCARDAPIO";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "IDPRATOESPE" => $cardLib,
            "NRCARDAPIO" => $nrcardapio
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    private function insertCompCardDel($cardapio, $prato, $idlocalexcl) {
        $query = "INSERT INTO COMPCARDDEL (
                    CDFILIAL,
                    CDSERVFILI,
                    DTPRATEXCL,
                    NRSEQPRATEXCL,
                    IDLOCALEXCL,
                    CDFILAUXPRAT,
                    CDPRATO
                ) VALUES (
                    :CDFILIAL,
                    :CDSERVFILI,
                    :DTPRATEXCL,
                    :NRSEQPRATEXCL,
                    :IDLOCALEXCL,
                    :CDFILAUXPRAT,
                    :CDPRATO
                )";
        if(!empty($prato['CDFILAUXPRATMAT']) && !empty($prato['CDPRATOMAT'])) {
            $params = array(
                'CDFILIAL' => $cardapio['CDFILIAL'],
                'CDSERVFILI' => $cardapio['CDSERVFILI'],
                'DTPRATEXCL' => $cardapio['DTCARDAPIO'],
                'NRSEQPRATEXCL' => $prato['NRSEQPRATCAR'],
                'IDLOCALEXCL' => $idlocalexcl,
                'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                'CDPRATO' => $prato['CDPRATO']
            );
        } else {
            $params = array(
                'CDFILIAL' => $cardapio['CDFILIAL'],
                'CDSERVFILI' => $cardapio['CDSERVFILI'],
                'DTPRATEXCL' => $cardapio['DTCARDAPIO'],
                'NRSEQPRATEXCL' => $prato['NRSEQPRATCAR'],
                'IDLOCALEXCL' => $idlocalexcl,
                'CDFILAUXPRAT' => $prato['CDFILAUXPRATMAT'],
                'CDPRATO' => $prato['CDPRATOMAT']
            );
        }
        $this->entityManager->getConnection()->prepare($query)->execute($params);
    }
    
    private function getPratosCard($cdfilial, $nrcardapio) {
        $query = "SELECT
                    NRSEQPRATCAR,
                    IDPRATOESPE,
                    CDFILAUXPRAT,
                    CDPRATO,
                    CDFILAUXPRATMAT,
                    CDPRATOMAT
                FROM
                    COMPCARD
                WHERE
                    CDFILIAL =:CDFILIAL AND
                    NRCARDAPIO =:NRCARDAPIO";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRCARDAPIO" => $nrcardapio
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    private function verificaTipoCardapioPorFilial($cdfilial) {
        $query = "SELECT
                    S.CDSERVFILI
                FROM
                    SERVICO S
                WHERE
                    S.CDFILIAL = :CDFILIAL AND
                    S.IDCARDDESCENTR = 'S'";
        $params = array(
            "CDFILIAL" => $cdfilial
        );
        $servicos = $this->entityManager->getConnection()->fetchAll($query, $params);
        $query = "SELECT
                    NVL(IDUTCARDPADR, 'N') AS IDUTCARDPADR
                FROM
                    PARAMGERAL";
        $idutcardpadr = $this->entityManager->getConnection()->fetchAll($query, [])[0]['IDUTCARDPADR'];
        if($idutcardpadr === 'S' && sizeof($servicos)) {
            return 'D';
        } else {
            return $idutcardpadr;
        }
    }
    
    private function getCentralCompras($cdfilial) {
        $query = "SELECT
                    CDFILCENTPME
                FROM
                    FILIAL
                WHERE
                    CDFILIAL = :CDFILIAL";
        $params = array(
            "CDFILIAL" => $cdfilial
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params)[0];
    }
    
    private function getCompCardDel($cdfilial, $cdservfili, $dtpratexcl, $nrseqpratexcl) {
        $query = "SELECT
                    CDFILIAL
                FROM
                    COMPCARDDEL
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTPRATEXCL = :DTPRATEXCL AND
                    NRSEQPRATEXCL = :NRSEQPRATEXCL";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili,
            "DTPRATEXCL" => $dtpratexcl,
            "NRSEQPRATEXCL" => $nrseqpratexcl
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    private function updateCompCardDel($cardapio, $prato, $idlocalexcl) {
        $query = "UPDATE
                    COMPCARDDEL
                SET
                    IDLOCALEXCL = :IDLOCALEXCL
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTPRATEXCL = :DTPRATEXCL AND
                    NRSEQPRATEXCL = :NRSEQPRATEXCL";
        $params = array(
            'IDLOCALEXCL' => $idlocalexcl,
            'CDFILIAL' => $cardapio['CDFILIAL'],
            'CDSERVFILI' => $cardapio['CDSERVFILI'],
            'DTPRATEXCL' => $cardapio['DTCARDAPIO'],
            'NRSEQPRATEXCL' => $prato['NRSEQPRATCAR']
        );
        $this->entityManager->getConnection()->prepare($query)->execute($params);
    }
    
    private function checkCardToDelete($modoProc, $rowCardapio, $tipoCardServ) {
        if($this->dependeLiberacao($tipoCardServ, $rowCardapio['CDFILIAL'], $rowCardapio['CDSERVFILI'])) {
            if($modoProc === 'M') { // matriz
                $cardLib = $this->verificaCardapioLiberado($rowCardapio);
                if($cardLib !== 'N') {
                    $pratos = $this->getPratosEspeCard($cardLib, $rowCardapio['CDFILIAL'], $rowCardapio['NRCARDAPIO']);
                    foreach ($pratos as $prato) {
                        $this->insertCompCardDel($rowCardapio, $prato, 'M');
                    }
                }
            } else { // filial
                $pratos = $this->getPratosCard($rowCardapio['CDFILIAL'], $rowCardapio['NRCARDAPIO']);
                foreach ($pratos as $prato) {
                    $this->insertCompCardDel($rowCardapio, $prato, 'F');
                }
            }
        }
        if($this->verificaTipoCardapioPorFilial($rowCardapio['CDFILIAL']) === 'S') {
            $rowCentCmp = getCentralCompras($rowCardapio['CDFILIAL']);
            if($rowCentCmp && $rowCentCmp['CDFILCENTPME']) {
                $pratos = $this->getPratosCard($rowCardapio['CDFILIAL'], $rowCardapio['NRCARDAPIO']);
                foreach ($pratos as $prato) {
                    if(sizeof($this->getCompCardDel($rowCardapio['CDFILIAL'], $rowCardapio['CDSERVFILI'], $rowCardapio['DTCARDAPIO'], $prato['NRSEQPRATCAR']))) {
                        $this->updateCompCardDel($rowCardapio, $prato, $modoProc);
                    } else {
                        $this->insertCompCardDel($rowCardapio, $prato, $modoProc);
                    }
                }
            }
        }
    }
    
    public function excluirCardadia($row) {

        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
        );

        $cardadia = $this->entityManager->getRepository('Planejamento\Entities\Cardadia')->findBy($params);

        if (isset($cardadia[0])) {

            $this->entityManager->remove($cardadia[0]);
            $this->entityManager->flush();

        }
    }

    public function excluirCardacli($row) {

        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
        );

        $cardacli = $this->entityManager->getRepository('Planejamento\Entities\Cardacli')->findBy($params);
        foreach ($cardacli as $cliente) {

          if (isset($cliente)) {

                $this->entityManager->remove($cliente);
                $this->entityManager->flush();

            }
        }

    }
    
    private function deleteCompCard($cardapio) {
        $query = "DELETE FROM 
                    COMPCARD
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    INSTR(:NRCARDAPIO, NRCARDAPIO) <> 0";
        $params = array(
            "CDFILIAL" => $cardapio['CDFILIAL'],
            "NRCARDAPIO" => $cardapio['NRCARDAPIO']
        );
        $this->entityManager->getConnection()->prepare($query)->execute($params);
    }
    
    public function verificaRetirada($cdfilial, $nrcardapio){
        $query = "SELECT * FROM RETIRADA
                  WHERE CDFILIAL = :CDFILIAL
                    AND NRCARDAPIO = :NRCARDAPIO";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRCARDAPIO" => $nrcardapio
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    private function verificaTipoCardapioPorServico($cdfilial, $cdservfili) {
            $allServ = array();
            $query = "SELECT 
                        S.CDSERVFILI
                    FROM 
                        SERVICO S
                    WHERE
                        S.CDFILIAL = :CDFILIAL AND
                        S.CDSERVFILI = :CDSERVFILI AND
                        S.IDCARDDESCENTR = 'S'";
            $params = array(
                "CDFILIAL" => $cdfilial,
                "CDSERVFILI" => $cdservfili
            );
            $servicos = $this->entityManager->getConnection()->fetchAll($query, $params);

            if(!empty($servicos))
                $allServ[] = $servicos;
        
        $query = "SELECT
                    NVL(IDUTCARDPADR, 'N') AS IDUTCARDPADR
                FROM
                    PARAMGERAL";
        $idutcardpadr = $this->entityManager->getConnection()->fetchAll($query, [])[0]['IDUTCARDPADR'];
        if($idutcardpadr === 'S' && sizeof($allServ)) {
            return 'D';
        } else {
            return $idutcardpadr;
        }
    }
    
    public function validaDistriuicaoCardapio ($cardapio){
        
        $cardapiosComDistribuicao;
        $result = [];
            
        $cardapiosComDistribuicao = $this->verificaDistribuicao($cardapio['CDFILIAL'], $cardapio['NRCARDAPIO']);
        
        if($cardapiosComDistribuicao){
            $result[] = array(
                "NRDISTCARD" => $cardapiosComDistribuicao[0]['NRDISTCARD'],
                "CDFILIAL" => $cardapiosComDistribuicao[0]['CDFILIAL'],
                "CDSERVFILI" => $cardapiosComDistribuicao[0]['CDSERVFILI'],
                "NRCARDAPIO" => $cardapiosComDistribuicao[0]['NRCARDAPIO'],
                "DTCARDAPIO" => $cardapiosComDistribuicao[0]['DTCARDAPIO']
                );
        }

        return isset($result) ? $result : "" ;
    }
    
    public function verificaDistribuicao($cdfilial , $nrcardapio){
        $query = "SELECT DIS.NRDISTCARD, CAR.CDFILIAL, CAR.NRCARDAPIO, 
                         CAR.CDSERVFILI, TO_CHAR(CAR.DTCARDAPIO, 'DD/MM/YYYY') AS DTCARDAPIO FROM CARDADIA CAR
                LEFT JOIN  DISTCARD DIS
                    ON CAR.CDFILIAL = DIS.CDFILIAL
                    AND CAR.NRCARDAPIO = DIS.NRCARDAPIO
                WHERE DIS.CDFILIAL = :CDFILIAL
                    AND DIS.NRCARDAPIO = :NRCARDAPIO";
        $params = array(
            "CDFILIAL"   => $cdfilial,
            "NRCARDAPIO" => $nrcardapio
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
        
    }
    
    public function sendMail($dadosCardapio, $emailMessages, $status) {
        try {
            $smtp = new UtilSMTP($this->entityManager->getConnection(), $this->environment);
            $smtpParams = $smtp->findSmtpParamsUnit($this->environment->getCdFilial());
    
            if(count($smtpParams) === 0) {
                throw new \Exception('SMTPHOST_NOT_INF#' . $this->environment->getCdFilial(), 400);
            }
     
            $this->smtpService = new SMTPService($smtpParams);
            
            $paramServicoOper = $this->getParamServicoOper($dadosCardapio);
            $emailOperador = $this->getEmailOperMymeals($dadosCardapio['CDOPERULTATU']);
            
            $emails = array($paramServicoOper["DSEMAILNOTIF"], $emailOperador['DSEMAILOPER']);
            $nmOperadorAprovacao = $this->getNameOperatorApproval($this->environment->getCdOperador());
    
            $mailBody = $this->buildMail($dadosCardapio, $paramServicoOper, $emailMessages, $nmOperadorAprovacao, $status);
            $this->smtpService->setFromName($emailMessages["emailFromName"]);
            $this->smtpService->setSubject($emailMessages["emailSubject"]);
            $this->smtpService->sendMail($mailBody, $emails);
        }catch(\Exception $e) {
            throw new \Exception('ERROR_EMAIL_MYMEALS', 400);
        }
    }
    
    public function getNameOperatorApproval($cdoperador) {
        $sql = "SELECT NMOPERADOR FROM OPERADOR
                WHERE CDOPERADOR = :CDOPERADOR
                    AND NRORG = :NRORG";
        $params = array(
            "CDOPERADOR"   => $cdoperador,
            "NRORG" => $this->environment->getNrOrg()
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
    }
    
    public function getParamServicoOper($dadosCardapio) {
        $sql = "SELECT * FROM PARAMSERVICOOPER
                WHERE CDFILIAL = :CDFILIAL
                    AND CDSERVFILI = :CDSERVFILI
                    AND NRORG = :NRORG";
        $params = array(
            "CDFILIAL"   => $dadosCardapio['CDFILIAL'],
            "CDSERVFILI"   => $dadosCardapio['CDSERVFILI'],
            "NRORG" => $this->environment->getNrOrg()
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
    }

    
    public function getEmailOperMymeals($cdoperadorMymeals){
        $sql = "SELECT DSEMAILOPER FROM OPERADOR
                WHERE CDOPERADOR = :CDOPERADOR
                    AND NRORG = :NRORG";
        $params = array(
            "CDOPERADOR"   => $cdoperadorMymeals,
            "NRORG" => $this->environment->getNrOrg()
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
        
    }
    
    public function buildMail ($dadosCardapio, $paramServicoOper, $emailMessages, $nmOperadorAprovacao, $status) {
        $title = $emailMessages["emailDescription"];
        $content = $this->fileGetContentUtf8($this->templatesDir."mail.html");
        $dtcardapio = substr($dadosCardapio["DTCARDAPIO"], 0, 10);
        $nmoperador = $dadosCardapio['NMOPERADOR'];
        $nmfilial = $dadosCardapio['NMFILIAL'];
        $footerMessage = $emailMessages["emailFooter"];
        $justification = array_key_exists('JUSTIFICATIVA', $dadosCardapio) ? $dadosCardapio['JUSTIFICATIVA'] : null;

        $search = array(
            "%title%",
            "%dateLabel%",
            "%dateValue%",
            "%operatorLabel%",
            "%operatorValue%",
            "%unitLabel%",
            "%unitValue%",
            "%serviceLabel%",
            "%serviceValue%",
            "%originalAmountLabel%",
            "%originalAmountValue%",
            "%changedAmountLabel%",
            "%changedAmountValue%",
            "%statusLabel%",
            "%statusValue%",
            "%justificationLabel%",
            "%justification%",
            "%operatorApprovalLabel%",
            "%operatorApproval%",
            "%footerMessage%"
        );
        $replace = array(
            $title,
            $emailMessages["emailDataCardapio"],
            $dtcardapio,
            $emailMessages["emailOperador"],
            $nmoperador,
            $emailMessages["emailUnidade"],
            $nmfilial,
            $emailMessages["emailServico"],
            $dadosCardapio["NMSERVICO"],
            $emailMessages["emailQtOri"],
            $dadosCardapio["QTCOMEORIGINAL"],
            $emailMessages["emailQtAlt"],
            $dadosCardapio["QTCOMEALTERADA"],
            $emailMessages["emailSituacao"],
            $emailMessages[$status],
            $emailMessages["emailJustificationLabel"],
            $justification,
            $emailMessages["emailOperadorAprovacaoLabel"],
            $nmOperadorAprovacao['NMOPERADOR'],
            $footerMessage
            
        );
        return str_replace($search, $replace, $content);
    }
    
    public function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }
    
}